import functools
import textwrap

import perf


IMPLEMENTATIONS = {
    'c': """\
    from multidict._multidict import istr
    """,
    'python': """\
    from multidict._multidict_py import istr
    """
}

INIT = """\
val = istr('VaLuE')
"""


ISTR_TO_ISTR = """\
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
istr(val)
"""


def benchmark_name(name, ctx, prefix=None, use_prefix=False):
    if use_prefix:
        return '%s%s' % (prefix % ctx, name)

    return name


def add_impl_option(cmd, args):
    if args.impl:
        cmd.extend(['--impl', args.impl])


if __name__ == '__main__':
    runner = perf.Runner(add_cmdline_args=add_impl_option)

    parser = runner.argparser
    parser.description = ('Allows to measure performance of '
                          'istr implementations')
    parser.add_argument('--impl', choices=sorted(IMPLEMENTATIONS),
                        help='specific implementation to benchmark')

    options = parser.parse_args()
    implementations = (options.impl,) if options.impl else IMPLEMENTATIONS

    for name in implementations:
        imports = textwrap.dedent(IMPLEMENTATIONS[name])
        name = functools.partial(benchmark_name, ctx=dict(impl=name),
                                 prefix='(impl = %(impl)s) ',
                                 use_prefix=len(implementations) > 1)

        runner.timeit(name('istr->istr'),
                      ISTR_TO_ISTR, imports + INIT,
                      inner_loops=10)



#!/usr/bin/env python3

import sys
from pathlib import Path


ALLOWED_SUFFIXES = ['.feature',
                    '.bugfix',
                    '.doc',
                    '.removal',
                    '.misc']


def get_root(script_path):
    folder = script_path.resolve().parent
    while not (folder / '.git').exists():
        folder = folder.parent
        if folder == folder.anchor:
            raise RuntimeError("git repo not found")
    return folder


def main(argv):
    print('Check "CHANGES" folder... ', end='', flush=True)
    here = Path(argv[0])
    root = get_root(here)
    changes = root / 'CHANGES'
    failed = False
    for fname in changes.iterdir():
        if fname.name in ('.gitignore', '.TEMPLATE.rst'):
            continue
        if fname.suffix not in ALLOWED_SUFFIXES:
            if not failed:
                print('')
            print(fname, 'has illegal suffix', file=sys.stderr)
            failed = True

    if failed:
        print('', file=sys.stderr)
        print('Allowed suffixes are:', ALLOWED_SUFFIXES, file=sys.stderr)
        print('', file=sys.stderr)
    else:
        print('OK')

    return int(failed)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



import functools
import textwrap

import pyperf


IMPLEMENTATIONS = {
    'dict': """\
    cls = dict
    """,
    'multidict_c': """\
    from multidict._multidict import MultiDict as cls, istr
    """,
    'cimultidict_c': """\
    from multidict._multidict import CIMultiDict as cls, istr
    """,
    'multidict_py': """\
    from multidict._multidict_py import MultiDict as cls, istr
    """,
    'cimultidict_py': """\
    from multidict._multidict_py import CIMultiDict as cls, istr
    """
}

INIT = """\
dct = cls()
"""

FILL = """\
for i in range(20):
    dct['key'+str(i)] = str(i)

key = 'key10'
"""


FILL_ISTR = """\
for i in range(20):
    key = istr('key'+str(i))
    dct[key] = str(i)

key = istr('key10')
"""


SET_ITEM = """\
dct[key] = '1'
dct[key] = '2'
dct[key] = '3'
dct[key] = '4'
dct[key] = '5'
dct[key] = '6'
dct[key] = '7'
dct[key] = '8'
dct[key] = '9'
dct[key] = '10'
""" * 10


GET_ITEM = """\
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
dct[key]
""" * 10


ADD = """\
add(key, '1')
add(key, '2')
add(key, '3')
add(key, '4')
add(key, '5')
add(key, '6')
add(key, '7')
add(key, '8')
add(key, '9')
add(key, '10')
""" * 10

SETUP_ADD = """\
add = dct.add
"""

def benchmark_name(name, ctx, prefix=None, use_prefix=False):
    if use_prefix:
        return '%s%s' % (prefix % ctx, name)

    return name


def add_impl_option(cmd, args):
    if args.impl:
        cmd.extend(['--impl', args.impl])


if __name__ == '__main__':
    runner = pyperf.Runner(add_cmdline_args=add_impl_option)

    parser = runner.argparser
    parser.description = ('Allows to measure performance of MultiMapping and '
                          'MutableMultiMapping implementations')
    parser.add_argument('--impl', choices=sorted(IMPLEMENTATIONS),
                        help='specific implementation to benchmark')

    options = parser.parse_args()
    implementations = (options.impl,) if options.impl else IMPLEMENTATIONS
    inner_loops = 50

    for impl in implementations:
        # print("=======================", impl, "======================")
        imports = textwrap.dedent(IMPLEMENTATIONS[impl])
        name = functools.partial(benchmark_name, ctx=dict(impl=impl),
                                 prefix='(impl = %(impl)s) ',
                                 use_prefix=len(implementations) > 1)

        runner.timeit(name('setitem str'),
                      SET_ITEM, imports + INIT + FILL,
                      inner_loops=inner_loops)
        runner.timeit(name('getitem str'),
                      GET_ITEM, imports + INIT + FILL,
                      inner_loops=inner_loops)

        # MultiDict specific
        if impl == 'dict':
            continue

        runner.timeit(name('setitem istr'),
                      SET_ITEM, imports + INIT + FILL_ISTR,
                      inner_loops=inner_loops)
        runner.timeit(name('getitem istr'),
                      GET_ITEM, imports + INIT + FILL_ISTR,
                      inner_loops=inner_loops)
        runner.timeit(name('add str'),
                      ADD, imports + INIT + FILL + SETUP_ADD,
                      inner_loops=inner_loops)
        runner.timeit(name('add istr'),
                      ADD, imports + INIT + FILL_ISTR + SETUP_ADD,
                      inner_loops=inner_loops)



import codecs
import os
import platform
import re
import sys

from setuptools import Extension, setup

NO_EXTENSIONS = bool(os.environ.get("MULTIDICT_NO_EXTENSIONS"))

if sys.implementation.name != "cpython":
    NO_EXTENSIONS = True

CFLAGS = ["-O2"]
# CFLAGS = ['-g']
if platform.system() != "Windows":
    CFLAGS.extend(
        [
            "-std=c99",
            "-Wall",
            "-Wsign-compare",
            "-Wconversion",
            "-fno-strict-aliasing",
            "-pedantic",
        ]
    )

extensions = [
    Extension(
        "multidict._multidict",
        ["multidict/_multidict.c"],
        extra_compile_args=CFLAGS,
    ),
]


with codecs.open(
    os.path.join(
        os.path.abspath(os.path.dirname(__file__)), "multidict", "__init__.py"
    ),
    "r",
    "latin1",
) as fp:
    try:
        version = re.findall(r'^__version__ = "([^"]+)"\r?$', fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError("Unable to determine version.")


def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()


args = dict(
    name="multidict",
    version=version,
    description=("multidict implementation"),
    long_description=read("README.rst"),
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Development Status :: 5 - Production/Stable",
    ],
    author="Andrew Svetlov",
    author_email="andrew.svetlov@gmail.com",
    url="https://github.com/aio-libs/multidict",
    project_urls={
        "Chat: Gitter": "https://gitter.im/aio-libs/Lobby",
        "CI: Azure Pipelines": "https://dev.azure.com/aio-libs/multidict/_build",
        "Coverage: codecov": "https://codecov.io/github/aio-libs/multidict",
        "Docs: RTD": "https://multidict.readthedocs.io",
        "GitHub: issues": "https://github.com/aio-libs/multidict/issues",
        "GitHub: repo": "https://github.com/aio-libs/multidict",
    },
    license="Apache 2",
    packages=["multidict"],
    python_requires=">=3.6",
    include_package_data=True,
)

if not NO_EXTENSIONS:
    print("*********************")
    print("* Accelerated build *")
    print("*********************")
    setup(ext_modules=extensions, **args)
else:
    print("*********************")
    print("* Pure Python build *")
    print("*********************")
    setup(**args)
